# django-salesforce
#
# by Phil Christensen
# (c) 2012-2013 Freelancers Union (http://www.freelancersunion.org)
# See LICENSE.md for details
#

"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import TestCase


# This dummy test is run in Django 1.6+ for:
#   $ python manage.py test salesforce
# but not run in previous versions until it is explicitely added:
#   $ python manage.py test salesforce example
# that is not valid in Django 1.6+
# Therefore it is removed for better consistency.

#class SimpleTest(TestCase):
#	def test_basic_addition(self):
#		"""
#		Tests that 1 + 1 always equals 2.
#		"""
#		self.assertEqual(1 + 1, 2)
