# django-salesforce
#
# by Phil Christensen
# (c) 2012-2013 Freelancers Union (http://www.freelancersunion.org)
# See LICENSE.md for details
#

"""
Default validation code.
"""

from django.db.backends import BaseDatabaseValidation

class DatabaseValidation(BaseDatabaseValidation):
	pass

